from datetime import datetime

from cloud_governance.policy.helpers.aws.aws_policy_operations import AWSPolicyOperations


class InstanceRun(AWSPolicyOperations):

    INSTANCE_TYPES_ES_INDEX = "cloud-governance-instance-types"
    RESOURCE_ACTION = "Stopped"

    def __init__(self):
        super().__init__()

    def _upload_instance_type_count_to_elastic_search(self):
        """
        This method uploads the instance type count to elasticsearch
        :return:
        :rtype:
        """
        instance_types = self._update_instance_type_count()
        account = self.account
        current_day = datetime.utcnow()
        es_instance_types_data = []
        for region, instance_types in instance_types.items():
            for instance_type, instance_type_count in instance_types.items():
                es_instance_types_data.append({
                    'instance_type': instance_type,
                    'instance_count': instance_type_count,
                    'timestamp': current_day,
                    'region': region,
                    'account': account,
                    'PublicCloud': self._cloud_name,
                    'index_id': f'{instance_type}-{self._cloud_name.lower()}-{account.lower()}-{region}-{str(current_day.date())}'
                })
        self._es_upload.es_upload_data(items=es_instance_types_data, es_index=self.INSTANCE_TYPES_ES_INDEX,
                                       set_index='index_id')

    def _update_instance_type_count(self):
        """
        This method returns the instance types count by region
        :return: { region: { instance_type: count } }
        :rtype: dict
        """
        instance_types = {}
        resources = self._get_all_instances()
        for instance in resources:
            instance_type = instance.get('InstanceType')
            instance_types.setdefault(self._region, {}).update(
                {instance_type: instance_types.get(self._region).get(instance_type, 0) + 1}
            )
        return instance_types

    def run_policy_operations(self):
        """
        This method returns the running instances
        :return:
        :rtype:
        """
        self._upload_instance_type_count_to_elastic_search()
        instances = self._get_all_instances()
        running_instances_data = []
        for instance in instances:
            tags = instance.get('Tags', [])
            cleanup_result = False
            if instance.get('State', {}).get('Name') == 'running':
                running_days = self.calculate_days(instance.get('LaunchTime'))
                if self._shutdown_period:
                    cleanup_days = self.get_clean_up_days_count(tags=tags)
                    cleanup_result = self.verify_and_delete_resource(
                        resource_id=instance.get('InstanceId'), tags=tags,
                        clean_up_days=cleanup_days)
                else:
                    cleanup_days = 0
                resource_data = self._get_es_schema(
                    resource_id=instance.get('InstanceId'),
                    skip_policy=self.get_skip_policy_value(tags=tags),
                    user=self.get_tag_name_from_tags(tags=tags, tag_name='User'),
                    launch_time=instance['LaunchTime'].strftime("%Y-%m-%dT%H:%M:%S+00:00"),
                    resource_type=instance.get('InstanceType'),
                    resource_state=instance.get('State', {}).get('Name') if not cleanup_result else 'stopped',
                    resource_action=self.RESOURCE_ACTION,
                    running_days=running_days, cleanup_days=cleanup_days,
                    dry_run=self._dry_run,
                    name=self.get_tag_name_from_tags(tags=tags, tag_name='Name'),
                    region=self._region, cleanup_result=str(cleanup_result),
                    cloud_name=self._cloud_name,
                )
                if self._shutdown_period and self._force_delete and self._dry_run == 'no':
                    resource_data.update({'ForceDeleted': str(self._force_delete)})
                running_instances_data.append(resource_data)
            else:
                cleanup_days = 0
            if self._shutdown_period:
                self.update_resource_day_count_tag(resource_id=instance.get('InstanceId'), cleanup_days=cleanup_days,
                                                   tags=tags)

        return running_instances_data



from abc import ABC, abstractmethod
from datetime import datetime
from typing import Union

from cloud_governance.common.elasticsearch.elastic_upload import ElasticUpload
from cloud_governance.common.utils.configs import INSTANCE_IDLE_CPU_PERCENTAGE, INSTANCE_IDLE_NETWORK_IN_KILO_BYTES, \
    INSTANCE_IDLE_NETWORK_OUT_KILO_BYTES
from cloud_governance.common.utils.utils import Utils
from cloud_governance.main.environment_variables import environment_variables


class AbstractPolicyOperations(ABC):

    DAYS_TO_NOTIFY_ADMINS = 2
    DAYS_TO_TRIGGER_RESOURCE_MAIL = 4
    DAILY_HOURS = 24
    CURRENT_DATE = datetime.utcnow().date().__str__()

    def __init__(self):
        self._environment_variables_dict = environment_variables.environment_variables_dict
        self.account = self._environment_variables_dict.get('account')
        self._days_to_take_action = self._environment_variables_dict.get('DAYS_TO_TAKE_ACTION')
        self._dry_run = self._environment_variables_dict.get('dry_run')
        self._policy = self._environment_variables_dict.get('policy')
        self._force_delete = self._environment_variables_dict.get('FORCE_DELETE')
        self._resource_id = self._environment_variables_dict.get('RESOURCE_ID')
        self._es_upload = ElasticUpload()
        self._shutdown_period = self._environment_variables_dict.get('SHUTDOWN_PERIOD')

    def calculate_days(self, create_date: Union[datetime, str], start_date: Union[datetime, str] = datetime.utcnow()):
        """
        This method returns the days
        :param start_date:
        :type start_date:
        :param create_date:
        :type create_date:
        :return:
        :rtype:
        """
        if isinstance(create_date, str):
            create_date = datetime.strptime(create_date.split('T')[0], "%Y-%M-%d")
        if isinstance(start_date, str):
            start_date = datetime.strptime(start_date, "%Y-%m-%d")
        start_date = start_date.date()
        days = start_date - create_date.date()
        return days.days

    def get_clean_up_days_count(self, tags: Union[list, dict]):
        """
        This method returns the cleanup days count
        :param tags:
        :type tags:
        :return:
        :rtype:
        """
        if self._dry_run == 'yes':
            return 0
        last_used_day = self.get_tag_name_from_tags(tags=tags, tag_name='DaysCount')
        if not last_used_day:
            return 1
        else:
            date, days = last_used_day.split('@')
            if date != str(self.CURRENT_DATE):
                return int(days) + 1
            return 1 if int(days) == 0 else int(days)

    @abstractmethod
    def get_tag_name_from_tags(self, tags: Union[list, dict], tag_name: str):
        """
        This method returns the tag_value from the tags
        :param tags:
        :type tags:
        :param tag_name:
        :type tag_name:
        :return:
        :rtype:
        """
        raise NotImplementedError("This method is Not yet implemented")

    def get_skip_policy_value(self, tags: Union[list, dict]) -> str:
        """
        This method returns the skip value
        :param tags:
        :type tags:
        :return:
        :rtype:
        """
        policy_value = self.get_tag_name_from_tags(tags=tags, tag_name='Policy').strip()
        if not policy_value:
            policy_value = self.get_tag_name_from_tags(tags=tags, tag_name='Skip').strip()
        if policy_value:
            return policy_value.replace('_', '').replace('-', '').upper()
        return 'NA'

    @abstractmethod
    def _delete_resource(self, resource_id: str):
        """
        This method deletes the resource
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        raise NotImplementedError("This method is Not yet implemented")

    @abstractmethod
    def update_resource_day_count_tag(self, resource_id: str, cleanup_days: int, tags: list):
        """
        This method updates the resource tags
        :param resource_id:
        :type resource_id:
        :param cleanup_days:
        :type cleanup_days:
        :param tags:
        :type tags:
        :return:
        :rtype:
        """
        raise NotImplementedError("This method is Not yet implemented")

    def verify_and_delete_resource(self, resource_id: str, tags: Union[list, dict], clean_up_days: int,
                                   days_to_delete_resource: int = None, **kwargs):
        """
        This method verify and delete the resource by calculating the days
        :return:
        :rtype:
        """
        if self._resource_id == resource_id and self._force_delete and self._dry_run == 'no':
            self._delete_resource(resource_id=resource_id)
            return True
        if not days_to_delete_resource:
            days_to_delete_resource = self._days_to_take_action
        cleanup_resources = False
        if clean_up_days >= self._days_to_take_action - self.DAYS_TO_TRIGGER_RESOURCE_MAIL:
            if clean_up_days == self._days_to_take_action - self.DAYS_TO_TRIGGER_RESOURCE_MAIL:
                kwargs['delta_cost'] = kwargs.get('extra_purse')
                # @Todo, If it require add email alert. May In future will add the email alert.
            else:
                if clean_up_days >= days_to_delete_resource:
                    if self._dry_run == 'no':
                        if self.get_skip_policy_value(tags=tags) not in ('NOTDELETE', 'SKIP'):
                            self._delete_resource(resource_id=resource_id)
                            cleanup_resources = True
        return cleanup_resources

    @abstractmethod
    def _update_tag_value(self, tags: Union[list, dict], tag_name: str, tag_value: str):
        """
        This method returns the updated tag_list by adding the tag_name and tag_value to the tags
        :param tags:
        :type tags:
        :param tag_name:
        :type tag_name:
        :param tag_value:
        :type tag_value:
        :return:
        :rtype:
        """
        raise NotImplementedError("This method is Not yet implemented")

    @abstractmethod
    def _get_all_instances(self):
        """
        This method returns all the instances
        :return:
        :rtype:
        """
        raise NotImplementedError("This method not yet implemented")

    @abstractmethod
    def _get_all_volumes(self):
        """
        This method returns all the volumes
        :return:
        :rtype:
        """
        raise NotImplementedError("This method not yet implemented")

    def __current_savings_year(self, unit_price: float):
        """
        This method returns the savings this year
        :param unit_price:
        :type unit_price:
        :return:
        :rtype:
        """
        year_end_date = datetime.utcnow().date().replace(month=12, day=31)
        total_days = (year_end_date - datetime.utcnow().date()).days + 1
        return total_days * 24 * unit_price

    # ES Schema format

    def _get_es_schema(self, resource_id: str, user: str, skip_policy: str, cleanup_days: int, dry_run: str,
                       name: str,
                       region: str, cleanup_result: str, resource_action: str, cloud_name: str,
                       resource_state: str,
                       resource_type: str, **kwargs):
        current_date = datetime.utcnow().date()
        resource_data = {
            'ResourceId': resource_id,
            'User': user,
            'SkipPolicy': skip_policy,
            'ResourceType': resource_type,
            'ResourceState': resource_state,
            'CleanUpDays': cleanup_days,
            'DryRun': dry_run,
            'Name': name,
            'RegionName': region,
            f'Resource{resource_action}': cleanup_result,
            'PublicCloud': cloud_name,
            'ExpireDays': self._days_to_take_action,
            'UnitPrice': kwargs.get('unit_price', 0),
            'TotalYearlySavings': self.__current_savings_year(kwargs.get('unit_price', 0)),
            'index-id': f'{current_date}-{cloud_name.lower()}-{self.account.lower()}-{region.lower()}-{resource_id}-{resource_state.lower()}'
        }
        if kwargs.get('launch_time'):
            resource_data.update({'LaunchTime': kwargs.get('launch_time')})
        if kwargs.get('running_days'):
            resource_data.update({'RunningDays': kwargs.get('running_days')})
        if kwargs.get('volume_size'):
            resource_data.update({'VolumeSize': kwargs.get('volume_size')})
        if kwargs.get('create_date'):
            resource_data.update({'create_date': kwargs.get('create_date')})

        return resource_data

    @abstractmethod
    def run_policy_operations(self):
        """
        This method returns policy output
        :return:
        :rtype:
        """
        pass

    def run(self):
        """
        This method starts the policy operations
        :return:
        :rtype:
        """
        return self.run_policy_operations()

    def verify_instance_idle(self, resource_id: str):
        """
        This method returns the metrics of the resource
        :param resource_id:
        :type resource_id:
        :return:
        :rtype:
        """
        cpu_percentage = self.get_cpu_utilization_percentage_metric(resource_id)
        network_in_kilobytes = self.get_network_in_kib_metric(resource_id)
        network_out_kilobytes = self.get_network_out_kib_metric(resource_id)
        return Utils.less_than(cpu_percentage, INSTANCE_IDLE_CPU_PERCENTAGE) and \
            Utils.less_than(network_in_kilobytes, INSTANCE_IDLE_NETWORK_IN_KILO_BYTES) and \
            Utils.less_than(network_out_kilobytes, INSTANCE_IDLE_NETWORK_OUT_KILO_BYTES)

    def get_cpu_utilization_percentage_metric(self, resource_id: str, **kwargs):
        """
        This method returns the cpu utilization percentage
        :param resource_id:
        :type resource_id:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """
        raise NotImplementedError("Not implemented error")

    def get_network_in_kib_metric(self, resource_id: str, **kwargs):
        """
        This method returns the total Network In KiB
        :param resource_id:
        :type resource_id:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """
        raise NotImplementedError("Not implemented error")

    def get_network_out_kib_metric(self, resource_id: str, **kwargs):
        """
        This method returns the total Network Out KiB
        :param resource_id:
        :type resource_id:
        :param kwargs:
        :type kwargs:
        :return:
        :rtype:
        """
        raise NotImplementedError("Not implemented error")



from datetime import datetime, timedelta

import boto3

from cloud_governance.common.logger.init_logger import logger


class CostExplorerOperations:
    """
    This class extracts the price data from AWS Cost Explorer
    """

    START_DAY = 1
    END_DAY = 31
    DIMENSIONS = 'Dimensions'
    KEY = 'Key'
    VALUES = 'Values'
    PURCHASE_TYPE = 'PURCHASE_TYPE'
    SPOT_INSTANCES = 'Spot Instances'
    FILTER = 'Filter'
    PURCHASE_OPTIONS = ['On Demand Instances', 'Savings Plans', SPOT_INSTANCES, 'Standard Reserved Instances']
    CE_COST_TYPES = {'CHARGETYPE': 'RECORD_TYPE', 'PURCHASETYPE': PURCHASE_TYPE}
    CE_FILTER_TEMPLATE = {
        DIMENSIONS: {
            KEY: '',
            VALUES: []
        }
    }
    CE_COST_FILTERS = {'SPOT': {KEY: PURCHASE_TYPE, VALUES: [SPOT_INSTANCES]}}

    def __init__(self, ce_client=''):
        self.cost_explorer_client = boto3.client('ce') if not ce_client else ce_client

    def __get_ce_tag_filters(self, tag: str, ce_default_filter: dict):
        """
        This method returns the ce filter values
        :return:
        """
        if ':' in tag:
            tag, tag_filter = tag.split(':')
            tag_filter = tag_filter.upper()
            ce_filter = self.CE_FILTER_TEMPLATE
            if tag_filter in self.CE_COST_FILTERS:
                ce_filter[self.DIMENSIONS][self.KEY] = self.CE_COST_FILTERS[tag_filter][self.KEY]
                ce_filter[self.DIMENSIONS][self.VALUES] = self.CE_COST_FILTERS[tag_filter][self.VALUES]
            if ce_default_filter:
                ce_default_filter = {
                    'And': [
                        ce_default_filter,
                        ce_filter
                    ]
                }
            else:
                ce_default_filter = ce_filter
        return ce_default_filter

    def get_ce_report_filter_data(self, ce_response: dict, tag_name: str):
        """
        This method returns data filter by tag_name
        :param tag_name:
        :param ce_response:
        :return:
        """
        data = {}
        if ce_response.get('ResultsByTime'):
            for results_by_time in ce_response.get('ResultsByTime'):
                start_time = results_by_time.get('TimePeriod').get('Start')
                for group in results_by_time.get('Groups'):
                    name = group.get('Keys')[0].split('$')[-1].strip().replace(' ', '-') if group.get('Keys') else ''
                    amount = group.get('Metrics').get('UnblendedCost').get('Amount') if group.get('Metrics') else 0
                    index_id = "%s-%s" % (start_time, name)
                    data[index_id] = {
                        tag_name: amount,
                        'ce_match_id': index_id,
                        'start_date': start_time
                    }
        if ce_response.get('DimensionValueAttributes'):
            for dimension_values in ce_response.get('DimensionValueAttributes'):
                account_id = dimension_values.get("Value")
                account = dimension_values.get('Attributes').get('description')
                for key_index_id in data.keys():
                    if account_id in key_index_id:
                        index_id = f'{data[key_index_id]["start_date"]}-{account}'.lower()
                        data[key_index_id].update({'Account': account, 'index_id': index_id})
        return data

    def get_filter_data(self, ce_data: list, tag_name: str = '', group_by: bool = True):
        """
        This method filter the cost_explorer_data
        :param group_by:
        :param tag_name:
        :param ce_data:
        :return:
        """
        user_cost_response = {}
        if group_by:
            for data in ce_data:
                for cost_group in data.get('Groups'):
                    user = cost_group.get('Keys')[0].split('$')[-1]
                    user = user if user else 'REFUND'
                    user_cost = float(cost_group.get('Metrics').get('UnblendedCost').get('Amount'))
                    if user in user_cost_response:
                        user_cost_response[user]['Cost'] += user_cost
                    else:
                        user_cost_response[user] = {tag_name: user, 'Cost': user_cost}
            return list(user_cost_response.values())
        else:
            total_cost = 0
            for data in ce_data:
                total_cost += float(data.get('Total').get('UnblendedCost').get('Amount'))
            return total_cost

    def get_cost_by_tags(self, tag: str, granularity: str = 'DAILY', cost_metric: str = 'UnblendedCost',
                         start_date: str = '', end_date: str = '', **kwargs):
        """
        This method extracts the price by Tag provided
        @return:
        """
        if not start_date and not end_date:
            end_date = datetime.now() + timedelta(self.START_DAY)
            start_date = end_date - timedelta(self.END_DAY)
            start_date = str(start_date.strftime('%Y-%m-%d'))
            end_date = str(end_date.strftime('%Y-%m-%d'))
        if tag.upper() in self.CE_COST_TYPES:
            return self.get_cost_and_usage_from_aws(start_date=start_date, end_date=end_date, granularity=granularity,
                                                    GroupBy=[{'Type': 'DIMENSION', 'Key': self.CE_COST_TYPES[tag.upper()]}], **kwargs)
        else:
            kwargs[self.FILTER] = self.__get_ce_tag_filters(tag=tag, ce_default_filter=kwargs.get(self.FILTER))
            if ':' in tag:
                tag, tag_filter = tag.split(':')
            return self.get_cost_and_usage_from_aws(start_date=start_date, end_date=end_date, granularity=granularity,
                                                    cost_metric=cost_metric, GroupBy=[{'Type': 'TAG', 'Key': tag}], **kwargs)

    def get_cost_and_usage_from_aws(self, start_date: str, end_date: str, granularity: str = 'DAILY',
                                    cost_metric: str = 'UnblendedCost', **kwargs):
        """
        This method returns the cost and usage reports
        @param start_date:
        @param end_date:
        @param granularity:
        @param cost_metric:
        @param kwargs:
        @return:
        """
        try:
            if self.FILTER in kwargs and not kwargs.get('Filter'):
                kwargs.pop('Filter')
            usage_cost = {}
            response = self.cost_explorer_client.get_cost_and_usage(TimePeriod={
                'Start': start_date,
                'End': end_date
            }, Granularity=granularity, Metrics=[cost_metric], **kwargs)
            usage_cost['GroupDefinitions'] = response.get('GroupDefinitions')
            usage_cost['ResultsByTime'] = response.get('ResultsByTime')
            usage_cost['DimensionValueAttributes'] = response.get('DimensionValueAttributes')
            while response.get('NextPageToken'):
                response = self.cost_explorer_client.get_cost_and_usage(TimePeriod={
                    'Start': start_date,
                    'End': end_date
                }, Granularity=granularity, Metrics=[cost_metric], NextPageToken=response.get('NextPageToken'), **kwargs)
                usage_cost['ResultsByTime'].extend(response.get('ResultsByTime'))
                usage_cost['DimensionValueAttributes'].extend(response.get('DimensionValueAttributes'))
            return usage_cost
        except Exception as err:
            logger.error(err)
            return {
                'ResultsByTime': [],
                'DimensionValueAttributes': [],
                'GroupDefinitions': [],
            }

    def get_cost_forecast(self, start_date: str, end_date: str, granularity: str, cost_metric: str, **kwargs):
        """
        This method returns the cost forecasting
        @param start_date:
        @param end_date:
        @param granularity:
        @param cost_metric:
        @return:
        """
        try:
            return self.cost_explorer_client.get_cost_forecast(
                TimePeriod={
                    'Start': start_date,
                    'End': end_date
                },
                Granularity=granularity,
                Metric=cost_metric, **kwargs
            )
        except Exception as err:
            logger.error(err)
            return {'Total': {'Amount': 0}, 'ForecastResultsByTime': []}



import boto3


class STSOperations:
    """
    This class is responsible for STS (Security Token Service) operations
    """

    def __init__(self):
        self.sts_client = boto3.client('sts')

    def get_account_id(self):
        """
        This method returns the account_id
        @return:
        """
        return self.sts_client.get_caller_identity()['Account']



import logging
from datetime import datetime, timedelta

import typeguard

from cloud_governance.cloud_resource_orchestration.clouds.azure.resource_groups.cost_over_usage import CostOverUsage
from cloud_governance.cloud_resource_orchestration.clouds.common.abstract_collect_cro_reports import \
    AbstractCollectCROReports
from cloud_governance.common.logger.logger_time_stamp import logger_time_stamp
from cloud_governance.main.environment_variables import environment_variables


class CollectCROReports(AbstractCollectCROReports):
    """
    This method collects the user/instance-id data from the cost-explorer
    """

    def __init__(self):
        super().__init__()
        self.__cost_over_usage = CostOverUsage()
        self._account_id = self._environment_variables_dict.get('AZURE_SUBSCRIPTION_ID')
        self.__scope = f'subscriptions/{self._account_id}'

    def _get_account_budget_from_payer_ce_report(self):
        """
        This method returns the account budget from the payer ce reports
        Check policy cost_explorer_payer_billings
        :return:
        """
        query = {
            "query": {
                "bool": {
                    "must": [
                        {"term": {"CloudName.keyword": self._public_cloud_name}},
                        {"term": {"AccountId.keyword": self._account_id}},
                        {"term": {"Month": str(datetime.utcnow().year)}},
                    ]
                }
            },
            "size": 1
        }
        response = self._es_operations.fetch_data_by_es_query(query=query, es_index=self._ce_payer_index, search_size=1,
                                                              limit_to_size=True)
        if response:
            return response[0].get('_source').get(self.ALLOCATED_BUDGET)
        return 0

    @typeguard.typechecked
    @logger_time_stamp
    def get_user_cost_data(self, group_by_tag_name: str, group_by_tag_value: str, requested_date: datetime = '', forecast: bool = False, duration: int = 0, extra_filter_key_values: dict = None):
        """
        This method fetches data from the es_reports
        :param extra_filter_key_values:
        :param group_by_tag_value:
        :param group_by_tag_name:
        :param duration:
        :param forecast:
        :param requested_date:
        :return:
        """
        extra_filter_matches = [{'Tags': {'Key': group_by_tag_name, 'Values': [group_by_tag_value]}}]
        tags = {group_by_tag_name: group_by_tag_value}
        if extra_filter_key_values:
            tags.update({{filter_key: filter_value} for filter_key, filter_value in extra_filter_key_values.items()})
        start_date = requested_date.replace(minute=self.ZERO, hour=self.ZERO, second=self.ZERO, microsecond=self.ZERO)
        response = {}
        if forecast:
            # Todo Will Add in future release
            resource_type = 'Forecast'
            pass
        else:
            end_date = datetime.utcnow().replace(microsecond=self.ZERO) + timedelta(days=1)
            response = self.__cost_over_usage.get_monthly_user_es_cost_data(start_date=start_date, end_date=end_date,
                                                                            extra_matches=extra_filter_matches,
                                                                            extra_operation=self.AND,
                                                                            tag_name=group_by_tag_name, tags=tags)
            resource_type = 'Cost'
        if response:
            return round(response[self.ZERO].get(resource_type), self.DEFAULT_ROUND_DIGITS)
        return self.ZERO

    @typeguard.typechecked
    def _upload_cro_report_to_es(self, monitor_data: dict):
        """
        This method uploads the data to elastic search index and return the data
        :param monitor_data:
        :return:
        """
        upload_data = {}
        for ticket_id, instance_data in monitor_data.items():
            if instance_data:
                ticket_id = ticket_id.split('-')[-1]
                user = instance_data[self.ZERO].get('user')
                issue_description = self._jira_operations.get_issue_description(ticket_id=ticket_id, state='ANY')
                ticket_opened_date = issue_description.get('TicketOpenedDate')
                group_by_tag_name = self.TICKET_ID_VALUE
                user_cost = self.get_user_cost_data(group_by_tag_name=group_by_tag_name, group_by_tag_value=ticket_id,
                                                    requested_date=ticket_opened_date)
                cost_estimation = float(instance_data[self.ZERO].get('estimated_cost', self.ZERO))
                if self._es_operations.verify_elastic_index_doc_id(index=self.__cost_over_usage.es_index_cro,
                                                                   doc_id=ticket_id):
                    if self._check_value_in_es(tag_key='ticket_id_state', tag_value='in-progress', ticket_id=ticket_id):
                        es_data = self._es_operations.get_es_data_by_id(id=ticket_id, index=self.__cost_over_usage.es_index_cro)
                        es_data['_source']['ticket_opened_date'] = ticket_opened_date.date()
                        es_data['_source']['user'] = user
                        source = self._prepare_update_es_data(source=es_data.get('_source'), instance_data=instance_data, cost_estimation=cost_estimation, user_cost=user_cost)
                        self._es_operations.update_elasticsearch_index(index=self._es_index_cro, id=ticket_id, metadata=source)
                        upload_data[ticket_id] = source
                else:
                    if ticket_id not in upload_data:
                        source = self._prepare_instance_data(instance_data=instance_data, ticket_id=ticket_id,
                                                             cost_estimation=cost_estimation, user=user,
                                                             user_cost=user_cost, ticket_opened_date=ticket_opened_date)
                        source['ticket_opened_date'] = ticket_opened_date.date()
                        source['user'] = user
                        if not source.get(self.ALLOCATED_BUDGET):
                            source[self.ALLOCATED_BUDGET] = self._get_account_budget_from_payer_ce_report()
                        self.__cost_over_usage.es_operations.upload_to_elasticsearch(index=self._es_index_cro, data=source, id=ticket_id)
                        upload_data[ticket_id] = source
        return upload_data

    def _get_total_account_usage_cost(self):
        """
        This method returns the total account budget till date for this year
        :return:
        """
        current_date = datetime.utcnow()
        start_date = datetime(current_date.year, 1, 1, 0, 0, 0)
        end_date = current_date + timedelta(days=1)
        cost_explorer_operations = self.__cost_over_usage.get_cost_management_object()
        response = cost_explorer_operations.get_usage(scope=self.__scope, start_date=start_date, end_date=end_date,
                                                      granularity='Monthly')
        total_cost = cost_explorer_operations.get_total_cost(cost_data=response)
        return total_cost

    @logger_time_stamp
    def update_in_progress_ticket_cost(self):
        """
        This method updates the in-progress tickets costs
        :return:
        """
        query = {"query": {"bool": {"must": [
            {"term": {"cloud_name.keyword": self._public_cloud_name}},
            {"term": {"account_name.keyword": self._account_name.upper()}},
            {"term": {"ticket_id_state.keyword": "in-progress"}}
        ]
        }}}
        in_progress_es_tickets = self._es_operations.fetch_data_by_es_query(query=query, es_index=self._es_index_cro)
        total_account_cost = self._get_total_account_usage_cost()
        for in_progress_ticket in in_progress_es_tickets:
            source_data = in_progress_ticket.get('_source')
            ticket_id = source_data.get(self.TICKET_ID_KEY)
            if source_data.get('account_name').lower() in self._account_name.lower():
                ticket_opened_date = datetime.strptime(source_data.get('ticket_opened_date'), "%Y-%m-%d")
                group_by_tag_name = self.TICKET_ID_VALUE
                user_cost = self.get_user_cost_data(group_by_tag_name=group_by_tag_name, group_by_tag_value=ticket_id,
                                                    requested_date=ticket_opened_date)
                user_daily_cost = eval(source_data.get('user_daily_cost', "{}"))
                user_name = source_data.get('user')
                if not user_name:
                    user_name = source_data.get('user_cro')
                ce_user_daily_report = self.__get_user_daily_usage_report(days=4, group_by_tag_value=ticket_id,
                                                                          group_by_tag_name=group_by_tag_name,
                                                                          user_name=user_name)
                user_daily_cost.update(ce_user_daily_report)
                update_data = {'actual_cost': user_cost, 'timestamp': datetime.utcnow(),
                               f'TotalCurrentUsage-{datetime.utcnow().year}': total_account_cost,
                               'user_daily_cost': str(user_daily_cost)}
                if not source_data.get(self.ALLOCATED_BUDGET):
                    update_data[self.ALLOCATED_BUDGET] = self._get_account_budget_from_payer_ce_report()
                self._es_operations.update_elasticsearch_index(index=self._es_index_cro, metadata=update_data, id=ticket_id)

    def __get_user_daily_usage_report(self, days: int, group_by_tag_name: str, group_by_tag_value: str, user_name: str):
        """
        This method returns the users daily report from last X days
        :param days:
        :return:
        """
        user_daily_usage_report = {}
        self._get_user_usage_by_granularity(tag_name=group_by_tag_name, tag_value=group_by_tag_value,
                                            days=days, result_back_data=user_daily_usage_report)
        self._get_user_usage_by_granularity(tag_name='User', tag_value=user_name,
                                            days=days, result_back_data=user_daily_usage_report)
        return user_daily_usage_report

    def _get_user_usage_by_granularity(self, result_back_data: dict, tag_name: str, days: int, tag_value):
        """
        This method returns the organized input of the usage_reports
        :param result_back_data:
        :param tag_name:
        :param days:
        :param tag_value:
        :return:
        """
        end_date = datetime.utcnow()
        start_date = end_date - timedelta(days=days)
        cost_explorer_object = self.__cost_over_usage.get_cost_management_object()
        ce_daily_usage = cost_explorer_object.get_usage(scope=self.__scope, grouping=[tag_name], granularity='Daily',
                                                        start_date=start_date,
                                                        end_date=end_date, tags={tag_name: tag_value})
        filtered_ce_daily_usage = cost_explorer_object.get_prettify_data(cost_data=ce_daily_usage)
        for daily_cost in filtered_ce_daily_usage:
            start_date = daily_cost.get('UsageDate')
            if start_date:
                start_date = str(start_date)
                start_date = f'{start_date[0:4]}-{start_date[4:6]}-{start_date[6:]}'
            usage = round(float(daily_cost.get('Cost')), self.DEFAULT_ROUND_DIGITS)
            result_back_data.setdefault(start_date, {}).update({daily_cost.get('TagValue'): usage})



import ldap
# installation for rhel/centos - python3.9
# sudo dnf install -y python39-devel openldap-devel gcc

from cloud_governance.common.logger.init_logger import logger


class LdapSearch:

    def __init__(self, ldap_host_name: str):
        self.__ldap_client = ldap.initialize(f'ldap://{ldap_host_name}')

    def __get_manager_name(self, manager_data: str):
        """
        This method return manger name from the manager_data
        @param manager_data:
        @return:
        """
        try:
            manager_id = manager_data.replace('=', ':').split(',')[0].split(':')[-1]
            manager_details = self.__get_details(user_name=manager_id)
            return str(manager_details.get('cn')[0], 'UTF-8'), manager_id
        except Exception as err:
            return []

    def __organise_user_details(self, data: dict):
        """
        This method organise user details by fields
        @param data:
        @return:
        """
        try:
            user_data = {'displayName': str(data['displayName'][0], 'UTF-8'), 'FullName': str(data['cn'][0], 'UTF-8')}
            manager_name, manager_id = self.__get_manager_name(manager_data=str(data['manager'][0], 'UTF-8'))
            user_data['managerName'] = manager_name
            user_data['managerId'] = manager_id
            user_data['ManagerName'] = manager_name
            user_data['ManagerId'] = manager_id
            return user_data
        except Exception as err:
            return []

    def __get_details(self, user_name: str):
        """
        This method get the user data from ldap
        @param user_name:
        @return:
        """
        response = []
        try:
            self.__ldap_client.protocol_version = ldap.VERSION3
            self.__ldap_client.set_option(ldap.OPT_REFERRALS, 0)
            base = 'dc=redhat, dc=com'
            criteria = f'(&(uid={user_name}))'
            attributes = ['displayName', 'manager', 'cn']
            result = self.__ldap_client.search_s(base, ldap.SCOPE_SUBTREE, criteria, attributes)
            if len(result) == 1:
                response = result[0][1]
        except Exception as err:
            logger.info(err)
        return response

    def get_user_details(self, user_name):
        """
        This method return the ldap results and organize data
        @param user_name:
        @return:
        """
        user_data = self.__get_details(user_name=user_name)
        return self.__organise_user_details(data=user_data)
